@@ -1,19 +1,8 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2000 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "global.h"
@@ -21,14 +10,14 @@
 #include "faviconscache_p.h"
 
 #include <kprotocolinfo.h>
-#include <kconfig.h>
-#include <kconfiggroup.h>
+#include <KConfig>
+#include <KConfigGroup>
 #include <kfileitem.h>
-#include <klocalizedstring.h>
-#include <ksharedconfig.h>
-#include <kformat.h>
-#include <kfileutils.h>
-#include <qmimedatabase.h>
+#include <KLocalizedString>
+#include <KSharedConfig>
+#include <KFormat>
+#include <KFileUtils>
+#include <QMimeDatabase>
 #include <QUrl>
 
 #include "kiocoredebug.h"
