@@ -1,23 +1,11 @@
-/* This file is part of the KDE project
-   Copyright (C) 2000-2005 David Faure <faure@kde.org>
-   Copyright (C) 2007 Norbert Frese <nf2@scheinwelt.at>
-   Copyright (C) 2007 Thiago Macieira <thiago@kde.org>
-   Copyright (C) 2013-2014 Frank Reininghaus <frank78ac@googlemail.com>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 2000-2005 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2007 Norbert Frese <nf2@scheinwelt.at>
+    SPDX-FileCopyrightText: 2007 Thiago Macieira <thiago@kde.org>
+    SPDX-FileCopyrightText: 2013-2014 Frank Reininghaus <frank78ac@googlemail.com>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "udsentry.h"
@@ -46,7 +34,7 @@ public:
     int count() const;
     QString stringValue(uint udsField) const;
     long long numberValue(uint udsField, long long defaultValue = -1) const;
-#ifndef KIOCORE_NO_DEPRECATED
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 8)
     QList<uint> listFields() const;
 #endif
     QVector<uint> fields() const;
@@ -145,13 +133,13 @@ long long UDSEntryPrivate::numberValue(uint udsField, long long defaultValue) co
     return defaultValue;
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 8)
 QList<uint> UDSEntryPrivate::listFields() const
 {
     QList<uint> res;
     res.reserve(storage.size());
-    for (auto it = storage.cbegin(), end = storage.cend(); it != end; ++it) {
-        res.append(it->m_index);
+    for (const Field &field : storage) {
+        res.append(field.m_index);
     }
     return res;
 }
@@ -161,8 +149,8 @@ QVector<uint> UDSEntryPrivate::fields() const
 {
     QVector<uint> res;
     res.reserve(storage.size());
-    for (auto it = storage.cbegin(), end = storage.cend(); it != end; ++it) {
-        res.append(it->m_index);
+    for (const Field &field : storage) {
+        res.append(field.m_index);
     }
     return res;
 }
@@ -183,15 +171,14 @@ void UDSEntryPrivate::save(QDataStream &s) const
 {
     s << static_cast<quint32>(storage.size());
 
-    for (auto it = storage.cbegin(), end = storage.cend(); it != end; ++it)
-    {
-        uint uds = it->m_index;
+    for (const Field &field : storage) {
+        uint uds = field.m_index;
         s << uds;
 
         if (uds & KIO::UDSEntry::UDS_STRING) {
-            s << it->m_str;
+            s << field.m_str;
         } else if (uds & KIO::UDSEntry::UDS_NUMBER) {
-            s << it->m_long;
+            s << field.m_long;
         } else {
             Q_ASSERT_X(false, "KIO::UDSEntry", "Found a field with an invalid type");
         }
@@ -312,12 +299,12 @@ void UDSEntryPrivate::debugUDSEntry(QDebug &stream) const
 {
     QDebugStateSaver saver(stream);
     stream.nospace() << "[";
-    for (auto it = storage.cbegin(), end = storage.cend(); it != end; ++it) {
-        stream << " " << nameOfUdsField(it->m_index) << "=";
-        if (it->m_index & KIO::UDSEntry::UDS_STRING) {
-            stream << it->m_str;
-        } else if (it->m_index & KIO::UDSEntry::UDS_NUMBER) {
-            stream << it->m_long;
+    for (const Field &field : storage) {
+        stream << " " << nameOfUdsField(field.m_index) << "=";
+        if (field.m_index & KIO::UDSEntry::UDS_STRING) {
+            stream << field.m_str;
+        } else if (field.m_index & KIO::UDSEntry::UDS_NUMBER) {
+            stream << field.m_long;
         } else {
             Q_ASSERT_X(false, "KIO::UDSEntry", "Found a field with an invalid type");
         }
@@ -398,15 +385,19 @@ void UDSEntry::fastInsert(uint field, long long value)
     d->insert(field, value);
 }
 
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 48)
 void UDSEntry::insert(uint field, const QString &value)
 {
     d->replace(field, value);
 }
+#endif
 
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 48)
 void UDSEntry::insert(uint field, long long value)
 {
     d->replace(field, value);
 }
+#endif
 
 void UDSEntry::replace(uint field, const QString &value)
 {
@@ -418,7 +409,7 @@ void UDSEntry::replace(uint field, long long value)
     d->replace(field, value);
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 8)
 QList<uint> UDSEntry::listFields() const
 {
     return d->listFields();
@@ -463,3 +454,34 @@ KIOCORE_EXPORT QDataStream &operator>>(QDataStream &s, KIO::UDSEntry &a)
     a.d->load(s);
     return s;
 }
+
+KIOCORE_EXPORT bool operator==(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
+{
+    if (entry.count() != other.count()) {
+         return false;
+     }
+
+     const QVector<uint> fields = entry.fields();
+     for (uint field : fields) {
+         if (!other.contains(field)) {
+             return false;
+         }
+
+         if (field & UDSEntry::UDS_STRING) {
+             if (entry.stringValue(field) != other.stringValue(field)) {
+                 return false;
+             }
+         } else {
+             if (entry.numberValue(field) != other.numberValue(field)) {
+                 return false;
+             }
+         }
+     }
+
+     return true;
+}
+
+KIOCORE_EXPORT bool operator!=(const KIO::UDSEntry &entry, const KIO::UDSEntry &other)
+{
+    return !(entry == other);
+}
