@@ -1,21 +1,10 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2000-2005 David Faure <faure@kde.org>
-   Copyright (C) 2007 Norbert Frese <nf2@scheinwelt.at>
-   Copyright (C) 2007 Thiago Macieira <thiago@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000-2005 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2007 Norbert Frese <nf2@scheinwelt.at>
+    SPDX-FileCopyrightText: 2007 Thiago Macieira <thiago@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #ifndef UDSENTRY_H
@@ -341,6 +330,10 @@ public:
         /// @since 4.7.3
         UDS_INODE = 30 | UDS_NUMBER,
 
+        /// For folders, the recursize size of its content
+        /// @since 5.70
+        UDS_RECURSIVE_SIZE = 31 | UDS_NUMBER,
+
         /// Extra data (used only if you specified Columns/ColumnsTypes)
         /// NB: you cannot repeat this entry; use UDS_EXTRA + i
         /// until UDS_EXTRA_END.
