@@ -1,26 +1,14 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 1999,2000 Stephan Kulow <coolo@kde.org>
-                  1999,2000,2001,2002,2003 Carsten Pfeiffer <pfeiffer@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1999, 2000 Stephan Kulow <coolo@kde.org>
+    SPDX-FileCopyrightText: 1999, 2000, 2001, 2002, 2003 Carsten Pfeiffer <pfeiffer@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kdiroperator.h"
 #include <kprotocolmanager.h>
-#include <kiconloader.h>
+#include <KIconLoader>
 #include "kdirmodel.h"
 #include "kdiroperatordetailview_p.h"
 #include "kdiroperatoriconview_p.h"
@@ -29,7 +17,7 @@
 #include "kfilemetapreview_p.h"
 #include "kpreviewwidgetbase.h"
 #include "knewfilemenu.h"
-#include <kurlmimedata.h>
+#include <KUrlMimeData>
 #include "../pathhelpers_p.h"
 
 #include <config-kiofilewidgets.h>
@@ -50,9 +38,9 @@
 
 #include <kdirlister.h>
 #include <kfileitemdelegate.h>
-#include <klocalizedstring.h>
-#include <kmessagebox.h>
-#include <kjobwidgets.h>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KJobWidgets>
 #include <kio/deletejob.h>
 #include <kio/copyjob.h>
 #include <kio/jobuidelegate.h>
@@ -62,9 +50,9 @@
 #include <kfilepreviewgenerator.h>
 #include <krun.h>
 #include <kpropertiesdialog.h>
-#include <kactioncollection.h>
-#include <kconfiggroup.h>
-#include <ksharedconfig.h>
+#include <KActionCollection>
+#include <KConfigGroup>
+#include <KSharedConfig>
 
 template class QHash<QString, KFileItem>;
 
@@ -187,11 +175,14 @@ public:
     QUrl previewUrl;
     int previewWidth;
 
-    bool dirHighlighting;
+
     bool onlyDoubleClickSelectsFiles;
     bool followNewDirectories;
     bool followSelectedDirectories;
-    QString lastURL; // used for highlighting a directory on cdUp
+
+    bool dirHighlighting;
+    QUrl m_lastUrl; // used for highlighting a directory on back/cdUp
+
     QTimer *progressDelayTimer;
     int dropOptions;
 
@@ -229,10 +220,10 @@ KDirOperator::Private::Private(KDirOperator *_parent) :
     preview(nullptr),
     previewUrl(),
     previewWidth(0),
-    dirHighlighting(false),
     onlyDoubleClickSelectsFiles(!qApp->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick)),
     followNewDirectories(true),
     followSelectedDirectories(true),
+    dirHighlighting(true),
     progressDelayTimer(nullptr),
     dropOptions(0),
     actionMenu(nullptr),
@@ -924,7 +915,7 @@ void KDirOperator::setIconsZoom(int _value)
     const int maxSize = KIconLoader::SizeEnormous - KIconLoader::SizeSmall;
     const int val = (maxSize * value / 100) + KIconLoader::SizeSmall;
     d->itemView->setIconSize(QSize(val, val));
-    d->previewGenerator->updatePreviews();
+    d->previewGenerator->updateIcons();
 
     emit currentIconSizeChanged(value);
 }
@@ -1048,7 +1039,6 @@ void KDirOperator::setUrl(const QUrl &_newurl, bool clearforward)
         d->forwardStack.clear();
     }
 
-    d->lastURL = d->currUrl.toString(QUrl::StripTrailingSlash);
     d->currUrl = newurl;
 
     pathChanged();
@@ -1208,9 +1198,22 @@ void KDirOperator::back()
     d->forwardStack.push(new QUrl(d->currUrl));
 
     QUrl *s = d->backStack.pop();
-
-    setUrl(*s, false);
+    const QUrl newUrl = *s;
     delete s;
+
+    if (d->dirHighlighting) {
+        const QUrl _url =
+            newUrl.adjusted(QUrl::StripTrailingSlash).adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
+
+        if (_url == d->currUrl.adjusted(QUrl::StripTrailingSlash) && !d->backStack.isEmpty()) {
+            // e.g. started in a/b/c, cdUp() twice to "a", then back(), we highlight "c"
+            d->m_lastUrl = *(d->backStack.top());
+        } else {
+            d->m_lastUrl = d->currUrl;
+        }
+    }
+
+    setUrl(newUrl, false);
 }
 
 // Code pinched from kfm then hacked
@@ -1236,6 +1239,11 @@ void KDirOperator::cdUp()
 {
     // Allow /d/c// to go up to /d/ instead of /d/c/
     QUrl tmp(d->currUrl.adjusted(QUrl::NormalizePathSegments));
+
+    if (d->dirHighlighting) {
+        d->m_lastUrl = d->currUrl;
+    }
+
     setUrl(tmp.resolved(QUrl(QStringLiteral(".."))), true);
 }
 
@@ -1961,8 +1969,10 @@ void KDirOperator::setupActions()
 
     QAction *backAction = d->actionCollection->addAction(KStandardAction::Back, QStringLiteral("back"), this, SLOT(back()));
     backAction->setShortcut(Qt::Key_Backspace);
+    backAction->setToolTip(i18nc("@info", "Go back"));
 
-    d->actionCollection->addAction(KStandardAction::Forward, QStringLiteral("forward"), this, SLOT(forward()));
+    QAction* forwardAction = d->actionCollection->addAction(KStandardAction::Forward, QStringLiteral("forward"), this, SLOT(forward()));
+    forwardAction->setToolTip(i18nc("@info", "Go forward"));
 
     QAction *homeAction = d->actionCollection->addAction(KStandardAction::Home, QStringLiteral("home"), this, SLOT(home()));
     homeAction->setText(i18n("Home Folder"));
@@ -2508,6 +2518,10 @@ void KDirOperator::Private::_k_slotIOFinished()
     if (preview) {
         preview->clearPreview();
     }
+
+    if (dirHighlighting) {
+        parent->setCurrentItem(m_lastUrl);
+    }
 }
 
 void KDirOperator::Private::_k_slotCanceled()
