@@ -1,21 +1,9 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998-2009 David Faure <faure@kde.org>
-                 2003      Sven Leiber <s.leiber@web.de>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 1998-2009 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2003 Sven Leiber <s.leiber@web.de>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 or at your option version 3.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only
 */
 
 #include "knewfilemenu.h"
@@ -30,31 +18,34 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QMenu>
+#include <QPushButton>
 #include <QStandardPaths>
 
-#include <qtemporaryfile.h>
-#include <kactioncollection.h>
-#include <kconfiggroup.h>
+#include <QTemporaryFile>
+#include <KActionCollection>
+#include <KConfigGroup>
 #include <QDebug>
-#include <kdesktopfile.h>
-#include <kdirwatch.h>
+#include <KDesktopFile>
+#include <KDirWatch>
 #include <kdirnotify.h>
-#include <kjobwidgets.h>
-#include <klocalizedstring.h>
-#include <kmessagebox.h>
+#include <KJobWidgets>
+#include <KLocalizedString>
+#include <KMessageBox>
+#include <KMessageWidget>
 #include <kprotocolinfo.h>
 #include <kprotocolmanager.h>
 #include <krun.h>
-#include <kshell.h>
+#include <KShell>
 #include <kio/job.h>
 #include <kio/copyjob.h>
 #include <kio/jobuidelegate.h>
 #include <kio/fileundomanager.h>
 #include <kio/mkpathjob.h>
 #include <kurifilter.h>
+#include <KFileUtils>
 
 #include <kpropertiesdialog.h>
-#include <qmimedatabase.h>
+#include <QMimeDatabase>
 #ifdef Q_OS_WIN
 #include <sys/utime.h>
 #else
@@ -63,16 +54,17 @@
 
 static QString expandTilde(const QString &name, bool isfile = false)
 {
-    if (!name.isEmpty() && (!isfile || name[0] == QLatin1Char('\\'))) {
-        const QString expandedName = KShell::tildeExpand(name);
-        // When a tilde mark cannot be properly expanded, the above call
-        // returns an empty string...
-        if (!expandedName.isEmpty()) {
-            return expandedName;
-        }
+    if (name.isEmpty() || name == QLatin1Char('~')) {
+        return name;
     }
 
-    return name;
+    QString expandedName;
+    if (!isfile || name[0] == QLatin1Char('\\')) {
+        expandedName = KShell::tildeExpand(name);
+    }
+
+    // If a tilde mark cannot be properly expanded, KShell::tildeExpand returns an empty string
+    return !expandedName.isEmpty() ? expandedName : name;
 }
 
 // Singleton, with data shared by all KNewFileMenu instances
@@ -238,21 +230,13 @@ public:
 class KNewFileMenuPrivate
 {
 public:
-    explicit KNewFileMenuPrivate(KNewFileMenu *qq)
-        : m_menuItemsVersion(0),
-          m_modal(true),
-          m_viewShowsHiddenFiles(false),
-          m_firstFileEntry(nullptr),
+    explicit KNewFileMenuPrivate(KActionCollection *collection, KNewFileMenu *qq)
+        : m_actionCollection(collection),
           q(qq)
     {}
 
     bool checkSourceExists(const QString &src);
 
-    /**
-      * Asks user whether to create a hidden directory with a dialog
-      */
-    void confirmCreatingHiddenDir(const QString &name);
-
     /**
       * The strategy used for other desktop files than Type=Link. Example: Application, Device.
       */
@@ -301,7 +285,7 @@ public:
     /**
      * Callback function that reads in directory name from dialog and processes it
      */
-    void _k_slotCreateDirectory(bool writeHiddenDir = false);
+    void _k_slotCreateDirectory();
 
     /**
      * Callback function that reads in directory name from dialog and processes it. This will wirte
@@ -352,9 +336,16 @@ public:
     QDialog *m_fileDialog;
 
     KActionMenu *m_menuDev;
-    int m_menuItemsVersion;
-    bool m_modal;
+    int m_menuItemsVersion = 0;
     QAction *m_newDirAction;
+    KMessageWidget* m_messageWidget = nullptr;
+    QDialogButtonBox* m_buttonBox = nullptr;
+
+    // This is used to allow _k_slotTextChanged to know whether it's being used to
+    // create a file or a directory without duplicating code across two functions
+    bool m_creatingDirectory = false;
+    bool m_viewShowsHiddenFiles = false;
+    bool m_modal = true;
 
     /**
      * The action group that our actions belong to
@@ -371,9 +362,8 @@ public:
     QStringList m_supportedMimeTypes;
     QString m_tempFileToDelete; // set when a tempfile was created for a Type=URL desktop file
     QString m_text;
-    bool m_viewShowsHiddenFiles;
 
-    KNewFileMenuSingleton::Entry *m_firstFileEntry;
+    KNewFileMenuSingleton::Entry *m_firstFileEntry = nullptr;
 
     KNewFileMenu * const q;
 
@@ -391,10 +381,10 @@ bool KNewFileMenuPrivate::checkSourceExists(const QString &src)
         dialog->setModal(q->isModal());
         dialog->setAttribute(Qt::WA_DeleteOnClose);
 
-        QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
-        buttonBox->setStandardButtons(QDialogButtonBox::Ok);
+        m_buttonBox = new QDialogButtonBox(dialog);
+        m_buttonBox->setStandardButtons(QDialogButtonBox::Ok);
 
-        KMessageBox::createKMessageBox(dialog, buttonBox, QMessageBox::Warning,
+        KMessageBox::createKMessageBox(dialog, m_buttonBox, QMessageBox::Warning,
                                        i18n("<qt>The template file <b>%1</b> does not exist.</qt>", src),
                                        QStringList(), QString(), nullptr, KMessageBox::NoExec);
 
@@ -405,44 +395,6 @@ bool KNewFileMenuPrivate::checkSourceExists(const QString &src)
     return true;
 }
 
-void KNewFileMenuPrivate::confirmCreatingHiddenDir(const QString &name)
-{
-    if (!KMessageBox::shouldBeShownContinue(QStringLiteral("confirm_create_hidden_dir"))) {
-        _k_slotCreateHiddenDirectory();
-        return;
-    }
-
-    KGuiItem continueGuiItem(KStandardGuiItem::cont());
-    continueGuiItem.setText(i18nc("@action:button", "Create directory"));
-    KGuiItem cancelGuiItem(KStandardGuiItem::cancel());
-    cancelGuiItem.setText(i18nc("@action:button", "Enter a Different Name"));
-    cancelGuiItem.setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
-
-    QDialog *confirmDialog = new QDialog(m_parentWidget);
-    confirmDialog->setWindowTitle(i18n("Create hidden directory?"));
-    confirmDialog->setModal(m_modal);
-    confirmDialog->setAttribute(Qt::WA_DeleteOnClose);
-
-    QDialogButtonBox *buttonBox = new QDialogButtonBox(confirmDialog);
-    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    KGuiItem::assign(buttonBox->button(QDialogButtonBox::Ok), continueGuiItem);
-    KGuiItem::assign(buttonBox->button(QDialogButtonBox::Cancel), cancelGuiItem);
-
-    KMessageBox::createKMessageBox(confirmDialog, buttonBox, QMessageBox::Warning,
-                                   i18n("The name \"%1\" starts with a dot, so the directory will be hidden by default.", name),
-                                   QStringList(),
-                                   i18n("Do not ask again"),
-                                   nullptr,
-                                   KMessageBox::NoExec);
-
-    QObject::connect(buttonBox, SIGNAL(accepted()), q, SLOT(_k_slotCreateHiddenDirectory()));
-    QObject::connect(buttonBox, &QDialogButtonBox::rejected, q, &KNewFileMenu::createDirectory);
-
-    m_fileDialog = confirmDialog;
-    confirmDialog->show();
-
-}
-
 void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::Entry &entry)
 {
     if (!checkSourceExists(entry.templatePath)) {
@@ -462,7 +414,7 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
         const QUrl directory = mostLocalUrl(*it);
         const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + QLatin1Char('/') + KIO::encodeFileName(text));
         if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
-            text = KIO::suggestName(directory, text);
+            text = KFileUtils::suggestName(directory, text);
         }
 
         QUrl templateUrl;
@@ -497,40 +449,57 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
 void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entry &entry)
 {
     // The template is not a desktop file
-    // Show the small dialog for getting the destination filename
+    // Prompt the user to set the destination filename
     QString text = entry.text;
     text.remove(QStringLiteral("...")); // the ... is fine for the menu item but not for the default filename
     text = text.trimmed(); // In some languages, there is a space in front of "...", see bug 268895
+    // add the extension (from the templatePath), should work with .txt, .html and with ".tar.gz"... etc
+    const QString fileName = entry.templatePath.mid(entry.templatePath.lastIndexOf(QLatin1Char('/')));
+    const int dotIndex = fileName.indexOf(QLatin1Char('.'));
+    text += dotIndex > 0 ? fileName.midRef(dotIndex) : QStringRef();
+
     m_copyData.m_src = entry.templatePath;
 
     const QUrl directory = mostLocalUrl(m_popupFiles.first());
     const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + QLatin1Char('/') + KIO::encodeFileName(text));
     if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
-        text = KIO::suggestName(directory, text);
+        text = KFileUtils::suggestName(directory, text);
     }
 
     QDialog *fileDialog = new QDialog(m_parentWidget);
     fileDialog->setAttribute(Qt::WA_DeleteOnClose);
+    fileDialog->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     fileDialog->setModal(q->isModal());
+    m_fileDialog = fileDialog;
 
     QVBoxLayout *layout = new QVBoxLayout;
+    layout->setSizeConstraint(QLayout::SetFixedSize);
+
+    m_messageWidget = new KMessageWidget(fileDialog);
+    m_messageWidget->setCloseButtonVisible(false);
+    m_messageWidget->setWordWrap(true);
+    m_messageWidget->hide();
     QLabel *label = new QLabel(entry.comment, fileDialog);
 
     QLineEdit *lineEdit = new QLineEdit(fileDialog);
     lineEdit->setClearButtonEnabled(true);
     lineEdit->setText(text);
+    lineEdit->setMinimumWidth(400);
 
+    m_buttonBox = new QDialogButtonBox(fileDialog);
+    m_buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+    QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, fileDialog, &QDialog::accept);
+    QObject::connect(m_buttonBox, &QDialogButtonBox::rejected, fileDialog, &QDialog::reject);
+
+    m_creatingDirectory = false;
     _k_slotTextChanged(text);
     QObject::connect(lineEdit, SIGNAL(textChanged(QString)), q, SLOT(_k_slotTextChanged(QString)));
 
-    QDialogButtonBox *buttonBox = new QDialogButtonBox(fileDialog);
-    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    QObject::connect(buttonBox, &QDialogButtonBox::accepted, fileDialog, &QDialog::accept);
-    QObject::connect(buttonBox, &QDialogButtonBox::rejected, fileDialog, &QDialog::reject);
-
     layout->addWidget(label);
     layout->addWidget(lineEdit);
-    layout->addWidget(buttonBox);
+    layout->addWidget(m_buttonBox);
+    layout->addWidget(m_messageWidget);
+    layout->addStretch();
 
     fileDialog->setLayout(layout);
     QObject::connect(fileDialog, SIGNAL(accepted()), q, SLOT(_k_slotRealFileOrDir()));
@@ -714,7 +683,7 @@ void KNewFileMenuPrivate::fillMenu()
                             } else {
                                 mime = db.mimeTypeForName(entry.mimeType);
                             }
-                            Q_FOREACH (const QString &supportedMime, m_supportedMimeTypes) {
+                            for (const QString &supportedMime : qAsConst(m_supportedMimeTypes)) {
                                 if (mime.inherits(supportedMime)) {
                                     keep = true;
                                     break;
@@ -851,8 +820,14 @@ void KNewFileMenuPrivate::_k_slotActionTriggered(QAction *action)
 
 }
 
-void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
+void KNewFileMenuPrivate::_k_slotCreateDirectory()
 {
+    // Automatically trim trailing spaces since they're pretty much always
+    // unintentional and can cause issues on Windows in shared environments
+    while (m_text.endsWith(QLatin1Char(' '))) {
+        m_text.chop(1);
+    }
+
     QUrl url;
     QUrl baseUrl = m_popupFiles.first();
 
@@ -862,7 +837,7 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
         if (QDir::isAbsolutePath(name)) {
             url = QUrl::fromLocalFile(name);
         } else {
-            if (name == QLatin1String(".") || name == QLatin1String("..")) {
+            if (name == QLatin1Char('.') || name == QLatin1String("..")) {
                 KGuiItem enterNewNameGuiItem(KStandardGuiItem::ok());
                 enterNewNameGuiItem.setText(i18nc("@action:button", "Enter a Different Name"));
                 enterNewNameGuiItem.setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
@@ -872,11 +847,11 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
                 confirmDialog->setModal(m_modal);
                 confirmDialog->setAttribute(Qt::WA_DeleteOnClose);
 
-                QDialogButtonBox *buttonBox = new QDialogButtonBox(confirmDialog);
-                buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-                KGuiItem::assign(buttonBox->button(QDialogButtonBox::Ok), enterNewNameGuiItem);
+                m_buttonBox = new QDialogButtonBox(confirmDialog);
+                m_buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+                KGuiItem::assign(m_buttonBox->button(QDialogButtonBox::Ok), enterNewNameGuiItem);
 
-                KMessageBox::createKMessageBox(confirmDialog, buttonBox, QMessageBox::Critical,
+                KMessageBox::createKMessageBox(confirmDialog, m_buttonBox, QMessageBox::Critical,
                                    xi18nc("@info", "Could not create a folder with the name <filename>%1</filename><nl/>because it is reserved for use by the operating system.", name),
                                    QStringList(),
                                    QString(),
@@ -884,18 +859,13 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
                                    KMessageBox::NoExec,
                                    QString());
 
-                QObject::connect(buttonBox, &QDialogButtonBox::accepted, q, &KNewFileMenu::createDirectory);
+                m_creatingDirectory = true;
+                QObject::connect(m_buttonBox, &QDialogButtonBox::accepted, q, &KNewFileMenu::createDirectory);
                 m_fileDialog = confirmDialog;
                 confirmDialog->show();
                 _k_slotAbortDialog();
                 return;
             }
-            if (!m_viewShowsHiddenFiles && name.startsWith(QLatin1Char('.'))) {
-                if (!writeHiddenDir) {
-                    confirmCreatingHiddenDir(name);
-                    return;
-                }
-            }
             url = baseUrl;
             url.setPath(concatPaths(url.path(), name));
         }
@@ -925,7 +895,7 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
 
 void KNewFileMenuPrivate::_k_slotCreateHiddenDirectory()
 {
-    _k_slotCreateDirectory(true);
+    _k_slotCreateDirectory();
 }
 
 struct EntryWithName {
@@ -990,7 +960,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
         dir.setPath(path);
         const QStringList &entryList(dir.entryList(QStringList() << QStringLiteral("*.desktop"), QDir::Files));
         files.reserve(files.size() + entryList.size());
-        Q_FOREACH (const QString &entry, entryList) {
+        for (const QString &entry : entryList) {
             const QString file = concatPaths(dir.path(), entry);
             files.append(file);
         }
@@ -998,7 +968,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
 
     QMap<QString, KNewFileMenuSingleton::Entry> slist; // used for sorting
     QMap<QString, EntryWithName> ulist; // entries with unique URLs
-    Q_FOREACH (const QString &file, files) {
+    for (const QString &file : qAsConst(files)) {
         //qDebug() << file;
         if (file[0] != QLatin1Char('.')) {
             KNewFileMenuSingleton::Entry e;
@@ -1050,6 +1020,11 @@ void KNewFileMenuPrivate::_k_slotOtherDesktopFileClosed()
 
 void KNewFileMenuPrivate::_k_slotRealFileOrDir()
 {
+    // Automatically trim trailing spaces since they're pretty much always
+    // unintentional and can cause issues on Windows in shared environments
+    while (m_text.endsWith(QLatin1Char(' '))) {
+        m_text.chop(1);
+    }
     m_copyData.m_chosenFileName = m_text;
     _k_slotAbortDialog();
     executeStrategy();
@@ -1072,6 +1047,93 @@ void KNewFileMenuPrivate::_k_slotSymLink()
 
 void KNewFileMenuPrivate::_k_slotTextChanged(const QString &text)
 {
+    m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
+    // Validate input, displaying a KMessageWidget for questionable names
+
+    if (text.length() == 0) {
+        m_messageWidget->hide();
+        m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+    }
+
+    // Don't allow creating folders that would mask . or ..
+    else if (text == QLatin1Char('.') || text == QLatin1String("..")) {
+        m_messageWidget->setText(xi18nc("@info", "The name <filename>%1</filename> cannot be used because it is reserved for use by the operating system.", text));
+        m_messageWidget->setMessageType(KMessageWidget::Error);
+        m_messageWidget->animatedShow();
+        m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+    }
+
+    // File or folder would be hidden; show warning
+    else if (text.startsWith(QLatin1Char('.'))) {
+        m_messageWidget->setText(xi18nc("@info", "The name <filename>%1</filename> starts with a dot, so it will be hidden by default.", text));
+        m_messageWidget->setMessageType(KMessageWidget::Warning);
+        m_messageWidget->animatedShow();
+    }
+
+    // File or folder begins with a space; show warning
+    else if (text.startsWith(QLatin1Char(' '))) {
+        m_messageWidget->setText(xi18nc("@info", "The name <filename>%1</filename> starts with a space, which will result in it being shown before other items when sorting alphabetically, among other potential oddities.", text));
+        m_messageWidget->setMessageType(KMessageWidget::Warning);
+        m_messageWidget->animatedShow();
+    }
+
+#ifndef Q_OS_WIN
+    // Inform the user that slashes in folder names create a directory tree
+    else if (text.contains(QLatin1Char('/'))) {
+        if (m_creatingDirectory) {
+            QStringList folders = text.split(QLatin1Char('/'));
+            if (!folders.isEmpty()) {
+                if (folders.first().isEmpty()) {
+                    folders.removeFirst();
+                }
+            }
+            QString label;
+            if (folders.count() > 1) {
+                label = i18n("Using slashes in folder names will create sub-folders, like so:");
+                QString indentation = QString();
+                for (const QString &folder : qAsConst(folders)) {
+                    label.append(QLatin1Char('\n'));
+                    label.append(indentation);
+                    label.append(folder);
+                    label.append(QStringLiteral("/"));
+                    indentation.append(QStringLiteral("    "));
+                }
+            } else {
+                label = i18n("Using slashes in folder names will create sub-folders.");
+            }
+            m_messageWidget->setText(label);
+            m_messageWidget->setMessageType(KMessageWidget::Information);
+            m_messageWidget->animatedShow();
+        }
+    }
+#endif
+
+#ifdef Q_OS_WIN
+    // Slashes and backslashes are not allowed in Windows filenames; show error
+    else if (text.contains(QLatin1Char('/'))) {
+        m_messageWidget->setText(i18n("Slashes cannot be used in file and folder names."));
+        m_messageWidget->setMessageType(KMessageWidget::Error);
+        m_messageWidget->animatedShow();
+        m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+    }
+    else if (text.contains(QLatin1Char('\\'))) {
+        m_messageWidget->setText(i18n("Backslashes cannot be used in file and folder names."));
+        m_messageWidget->setMessageType(KMessageWidget::Error);
+        m_messageWidget->animatedShow();
+        m_buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+    }
+#endif
+
+    // Using a tilde to begin a file or folder name is not recommended
+    else if (text.startsWith(QLatin1Char('~'))) {
+        m_messageWidget->setText(i18n("Starting a file or folder name with a tilde is not recommended because it may be confusing or dangerous when using the terminal to delete things."));
+        m_messageWidget->setMessageType(KMessageWidget::Warning);
+        m_messageWidget->animatedShow();
+    }
+
+    else {
+        m_messageWidget->hide();
+    }
     m_text = text;
 }
 
@@ -1138,13 +1200,12 @@ void KNewFileMenuPrivate::_k_slotUrlDesktopFile()
 
 KNewFileMenu::KNewFileMenu(KActionCollection *collection, const QString &name, QObject *parent)
     : KActionMenu(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Create New"), parent),
-      d(new KNewFileMenuPrivate(this))
+      d(new KNewFileMenuPrivate(collection, this))
 {
     // Don't fill the menu yet
     // We'll do that in checkUpToDate (should be connected to aboutToShow)
     d->m_newMenuGroup = new QActionGroup(this);
     connect(d->m_newMenuGroup, SIGNAL(triggered(QAction*)), this, SLOT(_k_slotActionTriggered(QAction*)));
-    d->m_actionCollection = collection;
     d->m_parentWidget = qobject_cast<QWidget *>(parent);
     d->m_newDirAction = nullptr;
 
@@ -1170,9 +1231,7 @@ void KNewFileMenu::checkUpToDate()
         //qDebug() << "recreating actions";
         // We need to clean up the action collection
         // We look for our actions using the group
-        foreach (QAction *action, d->m_newMenuGroup->actions()) {
-            delete action;
-        }
+        qDeleteAll(d->m_newMenuGroup->actions());
 
         if (!s->templatesList) { // No templates list up to now
             s->templatesList = new KNewFileMenuSingleton::EntryList;
@@ -1210,38 +1269,49 @@ void KNewFileMenu::createDirectory()
                    d->m_text;
 
     if (baseUrl.isLocalFile() && QFileInfo::exists(baseUrl.toLocalFile() + QLatin1Char('/') + name)) {
-        name = KIO::suggestName(baseUrl, name);
+        name = KFileUtils::suggestName(baseUrl, name);
     }
 
     QDialog *fileDialog = new QDialog(d->m_parentWidget);
     fileDialog->setModal(isModal());
     fileDialog->setAttribute(Qt::WA_DeleteOnClose);
+    fileDialog->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
     fileDialog->setWindowTitle(i18nc("@title:window", "New Folder"));
+    d->m_fileDialog = fileDialog;
 
     QVBoxLayout *layout = new QVBoxLayout;
-    QLabel *label = new QLabel(i18n("Create new folder in:\n%1", baseUrl.toDisplayString(QUrl::PreferLocalFile)), fileDialog);
+    layout->setSizeConstraint(QLayout::SetFixedSize);
+
+    d->m_messageWidget = new KMessageWidget(fileDialog);
+    d->m_messageWidget->setCloseButtonVisible(false);
+    d->m_messageWidget->setWordWrap(true);
+    d->m_messageWidget->hide();
+    QLabel *label = new QLabel(i18n("Create new folder in %1:", baseUrl.toDisplayString(QUrl::PreferLocalFile)), fileDialog);
 
     QLineEdit *lineEdit = new QLineEdit(fileDialog);
     lineEdit->setClearButtonEnabled(true);
     lineEdit->setText(name);
+    lineEdit->setMinimumWidth(400);
 
+    d->m_buttonBox = new QDialogButtonBox(fileDialog);
+    d->m_buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+    QObject::connect(d->m_buttonBox, &QDialogButtonBox::accepted, fileDialog, &QDialog::accept);
+    QObject::connect(d->m_buttonBox, &QDialogButtonBox::rejected, fileDialog, &QDialog::reject);
+
+    d->m_creatingDirectory = true;
     d->_k_slotTextChanged(name); // have to save string in d->m_text in case user does not touch dialog
     connect(lineEdit, SIGNAL(textChanged(QString)), this, SLOT(_k_slotTextChanged(QString)));
 
-    QDialogButtonBox *buttonBox = new QDialogButtonBox(fileDialog);
-    buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    QObject::connect(buttonBox, &QDialogButtonBox::accepted, fileDialog, &QDialog::accept);
-    QObject::connect(buttonBox, &QDialogButtonBox::rejected, fileDialog, &QDialog::reject);
-
     layout->addWidget(label);
     layout->addWidget(lineEdit);
-    layout->addWidget(buttonBox);
+    layout->addWidget(d->m_buttonBox);
+    layout->addWidget(d->m_messageWidget);
+    layout->addStretch();
 
     fileDialog->setLayout(layout);
-    connect(fileDialog, SIGNAL(accepted()), this, SLOT(_k_slotCreateDirectory()));
+    connect(fileDialog, &QDialog::accepted, this, [this]() { d->_k_slotCreateDirectory(); });
     connect(fileDialog, SIGNAL(rejected()), this, SLOT(_k_slotAbortDialog()));
 
-    d->m_fileDialog = fileDialog;
 
     fileDialog->show();
     lineEdit->selectAll();
@@ -1283,7 +1353,7 @@ void KNewFileMenu::setPopupFiles(const QList<QUrl> &files)
     if (files.isEmpty()) {
         d->m_newMenuGroup->setEnabled(false);
     } else {
-        QUrl firstUrl = files.first();
+        const QUrl &firstUrl = files.first();
         if (KProtocolManager::supportsWriting(firstUrl)) {
             d->m_newMenuGroup->setEnabled(true);
             if (d->m_newDirAction) {
