@@ -1,25 +1,13 @@
 /*
-   Original Authors:
-   Copyright (c) Kalle Dalheimer <kalle@kde.org> 1997
-   Copyright (c) David Faure <faure@kde.org> 1998
-   Copyright (c) Dirk Mueller <mueller@kde.org> 2000
+    Original Authors:
+    SPDX-FileCopyrightText: 1997 Kalle Dalheimer <kalle@kde.org>
+    SPDX-FileCopyrightText: 1998 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2000 Dirk Mueller <mueller@kde.org>
 
-   Completely re-written by:
-   Copyright (C) 2000- Dawit Alemayehu <adawit@kde.org>
+    Completely re-written by:
+    SPDX-FileCopyrightText: 2000 Dawit Alemayehu <adawit@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License (GPL)
-   version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: GPL-2.0-only
 */
 
 // Own
@@ -37,15 +25,15 @@
 #include <QLoggingCategory>
 
 // KDE
-#include <kconfig.h>
+#include <KConfig>
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <kmessagebox.h>
+#include <KMessageBox>
 #include <http_slave_defaults.h>
-#include <kpluginfactory.h>
+#include <KPluginFactory>
 
 Q_DECLARE_LOGGING_CATEGORY(KIO_USERAGENTDLG)
-Q_LOGGING_CATEGORY(KIO_USERAGENTDLG, "kf5.kio.useragentdlg")
+Q_LOGGING_CATEGORY(KIO_USERAGENTDLG, "kf.configwidgets.cms.kf.kio.useragentdlg")
 
 K_PLUGIN_FACTORY_DECLARATION (KioConfigFactory)
 
@@ -176,7 +164,7 @@ void UserAgentDlg::changeDefaultUAModifiers()
     if (ui.languageCheckBox->isChecked())
         m_ua_keys += QLatin1Char('l');
 
-    ui.osVersionCheckBox->setEnabled (m_ua_keys.contains ('o'));
+    ui.osVersionCheckBox->setEnabled(m_ua_keys.contains(QLatin1Char('o')));
 
     QString modVal = KProtocolManager::defaultUserAgent (m_ua_keys);
     if (ui.defaultIdLineEdit->text() != modVal) {
@@ -268,10 +256,10 @@ void UserAgentDlg::load()
     ui.sendUACheckBox->setChecked (b);
     m_ua_keys = cg2.readEntry ("UserAgentKeys", DEFAULT_USER_AGENT_KEYS).toLower();
     ui.defaultIdLineEdit->setText (KProtocolManager::defaultUserAgent (m_ua_keys));
-    ui.osNameCheckBox->setChecked (m_ua_keys.contains ('o'));
-    ui.osVersionCheckBox->setChecked (m_ua_keys.contains ('v'));
-    ui.processorTypeCheckBox->setChecked (m_ua_keys.contains ('m'));
-    ui.languageCheckBox->setChecked (m_ua_keys.contains ('l'));
+    ui.osNameCheckBox->setChecked (m_ua_keys.contains(QLatin1Char('o')));
+    ui.osVersionCheckBox->setChecked (m_ua_keys.contains(QLatin1Char('v')));
+    ui.processorTypeCheckBox->setChecked (m_ua_keys.contains(QLatin1Char('m')));
+    ui.languageCheckBox->setChecked (m_ua_keys.contains(QLatin1Char('l')));
 
     updateButtons();
     configChanged (false);
@@ -282,10 +270,10 @@ void UserAgentDlg::defaults()
     ui.sitePolicyTreeWidget->clear();
     m_ua_keys = QStringLiteral(DEFAULT_USER_AGENT_KEYS);
     ui.defaultIdLineEdit->setText (KProtocolManager::defaultUserAgent (m_ua_keys));
-    ui.osNameCheckBox->setChecked (m_ua_keys.contains ('o'));
-    ui.osVersionCheckBox->setChecked (m_ua_keys.contains ('v'));
-    ui.processorTypeCheckBox->setChecked (m_ua_keys.contains ('m'));
-    ui.languageCheckBox->setChecked (m_ua_keys.contains ('l'));
+    ui.osNameCheckBox->setChecked (m_ua_keys.contains (QLatin1Char('o')));
+    ui.osVersionCheckBox->setChecked (m_ua_keys.contains (QLatin1Char('v')));
+    ui.processorTypeCheckBox->setChecked (m_ua_keys.contains (QLatin1Char('m')));
+    ui.languageCheckBox->setChecked (m_ua_keys.contains(QLatin1Char('l')));
     ui.sendUACheckBox->setChecked (true);
 
     updateButtons();
