@@ -1,21 +1,9 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-   Copyright (C) 2006 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 1998, 1999 Torben Weis <weis@kde.org>
+    SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KRUN_H
@@ -23,19 +11,21 @@
 
 #include "kiowidgets_export.h"
 
-#include <QtCore/QObject>
-#include <QtCore/QString>
-#include <QtCore/QUrl>
+#include <QObject>
+#include <QString>
+#include <QUrl>
 
 class KService;
 class KJob;
 class QTimer;
+class KRunPrivate;
 
 namespace KIO
 {
 class Job;
 }
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
 /**
  * @class KRun krun.h <KRun>
  *
@@ -81,8 +71,15 @@ public:
      * @param asn
      *        Application startup notification id, if available (otherwise "").
      *
-     * Porting note: kdelibs 4 had mode_t mode and bool isLocalFile arguments after
-     * window and before showProgressInfo. Removed in KF5.
+     * @deprecated since 5.71, use KIO::OpenUrlJob(url) (except for KRun subclasses, for now)
+     * @code
+     *   KIO::OpenUrlJob *job = new KIO::OpenUrlJob(url);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setStartupId(asn);
+     *   // methods like setRunExecutables, setSuggestedFilename, setEnableExternalBrowser, setFollowRedirections
+     *   // exist in both classes
+     *   job->start();
+     * @endcode
      */
     KRun(const QUrl &url, QWidget *window,
          bool showProgressInfo = true,
@@ -154,6 +151,14 @@ public:
      */
     void setRunExecutables(bool b);
 
+    /**
+     * Sets whether KRun should follow URLs redirections.
+     * This is enabled by default
+     * @param b whether to follow redirections or not.
+     * @since 5.55
+     */
+    void setFollowRedirections(bool b);
+
     /**
      * Sets whether the external webbrowser setting should be honoured.
      * This is enabled by default.
@@ -190,6 +195,7 @@ public:
      */
     QWidget *window() const;
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 6)
     /**
      * Open a list of URLs with a certain service (application).
      *
@@ -203,13 +209,26 @@ public:
      * @param asn Application startup notification id, if any (otherwise "").
      * @return @c true on success, @c false on error
      *
-     * @deprecated since 5.6, use runService instead. No change needed on the application side,
-     * the only difference is the return value (qint64 instead of bool).
+     * @deprecated since 5.6. Since 5.71 use ApplicationLauncherJob, otherwise runApplication instead.
+     * @code
+     *   KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service);
+     *   job->setUrls(urls);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   if (tempFiles) {
+     *       job->setRunFlags(KIO::ApplicationLauncherJob::DeleteTemporaryFiles);
+     *   }
+     *   job->setSuggestedFileName(suggestedFileName);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
-    static KIOWIDGETS_DEPRECATED bool run(const KService &service, const QList<QUrl> &urls, QWidget *window,
+    KIOWIDGETS_DEPRECATED_VERSION(5, 6, "Use KIO::ApplicationLauncherJob, see API docs for a code sample")
+    static bool run(const KService &service, const QList<QUrl> &urls, QWidget *window,
                     bool tempFiles = false, const QString &suggestedFileName = QString(),
                     const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Open a list of URLs with a certain service (application).
      *
@@ -227,22 +246,48 @@ public:
      * @param asn Application startup notification id, if any (otherwise "").
      * @return 0 on error, the process ID on success
      * @since 5.6
+     * @deprecated since 5.71, use ApplicationLauncherJob instead.
+     * @code
+     *   KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service);
+     *   job->setUrls(urls);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   if (tempFiles) {
+     *       job->setRunFlags(KIO::ApplicationLauncherJob::DeleteTemporaryFiles);
+     *   }
+     *   job->setSuggestedFileName(suggestedFileName);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::ApplicationLauncherJob, see API docs for a code sample")
     static qint64 runService(const KService &service, const QList<QUrl> &urls, QWidget *window,
                              bool tempFiles = false, const QString &suggestedFileName = QString(),
-                             const QByteArray &asn = QByteArray()); // TODO KF6: deprecate/remove
+                             const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
+    /**
+     * @see RunFlags
+     */
     enum RunFlag {
         DeleteTemporaryFiles = 0x1, ///< the URLs passed to the service will be deleted when it exits (if the URLs are local files)
         RunExecutables = 0x2,       ///< Whether to run URLs that are executable scripts or binaries @see isExecutableFile() @since 5.31
     };
+    /**
+     * Stores a combination of #RunFlag values.
+     * @deprecated since 5.71, see porting instructions in the respective methods
+     */
     Q_DECLARE_FLAGS(RunFlags, RunFlag)
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Run an application (known from its .desktop file, i.e. as a KService)
      *
-     * Unlike runService, this does not wait for the application to register to D-Bus
-     * before returning. Such behavior is better done with D-Bus activation anyway.
+     * If you need to wait for the application to register to D-Bus, use D-Bus activation instead.
+     *
+     * If you don't need the prompt for asking the user whether to add the executable bit for
+     * desktop files or binaries that don't have it, you can use KIO::ApplicationLauncherJob from KIOGui directly.
      *
      * @param service the service to run
      * @param urls the list of URLs, can be empty (app launched
@@ -253,11 +298,25 @@ public:
      * @param asn Application startup notification id, if any (otherwise "").
      * @return 0 on error, the process ID on success
      * @since 5.24
+     *
+     * @deprecated since 5.71, use ApplicationLauncherJob instead.
+     * @code
+     *   KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service);
+     *   job->setUrls(urls);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setSuggestedFileName(suggestedFileName);
+     *   job->setRunFlags(flags);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::ApplicationLauncherJob, see API docs for a code sample")
     static qint64 runApplication(const KService &service, const QList<QUrl> &urls, QWidget *window,
                                  RunFlags flags = RunFlags(), const QString &suggestedFileName = QString(),
                                  const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Open a list of URLs with an executable.
      *
@@ -273,12 +332,25 @@ public:
      * @param icon the icon which should be used by the application.
      * @param asn Application startup notification id, if any (otherwise "").
      * @return @c true on success, @c false on error
+     *
+     * @deprecated since 5.71, use KIO::ApplicationLauncherJob with a temporary KService
+     * @code
+     *   KService::Ptr service(new KService(name, exec, icon));
+     *   KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service);
+     *   job->setUrls(urls);
+     *   job->setUiDelegate(new KDialogJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::ApplicationLauncherJob with a temporary KService, see API docs for a code sample")
     static bool run(const QString &exec, const QList<QUrl> &urls, QWidget *window,
                     const QString &name = QString(),
                     const QString &icon = QString(),
                     const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 31)
     /**
      * Open the given URL.
      *
@@ -296,14 +368,24 @@ public:
      * @param suggestedFileName see setSuggestedFileName
      * @param asn Application startup notification id, if any (otherwise "").
      * @return @c true on success, @c false on error
-     * @deprecated since 5.31, use runUrl() with RunFlags instead.
+     * @deprecated since 5.31. Since 5.71 use OpenUrlJob, otherwise runUrl() with RunFlags.
+     * @code
+     *   KIO::OpenUrlJob *job = new KIO::OpenUrlJob(url, mimetype);
+     *   job->setSuggestedFileName(suggestedFileName);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setRunExecutables(runExecutables);
+     *   job->setDeleteTemporaryFile(...); // depending on the old RunFlags
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    static bool KIOWIDGETS_DEPRECATED runUrl(const QUrl &url, const QString &mimetype, QWidget *window,
-                                             bool tempFile = false, bool runExecutables = true,
-                                             const QString &suggestedFileName = QString(), const QByteArray &asn = QByteArray());
+    KIOWIDGETS_DEPRECATED_VERSION(5, 31, "Use KIO::OpenUrlJob, see API docs for a code sample")
+    static bool runUrl(const QUrl &url, const QString &mimetype, QWidget *window,
+                       bool tempFile = false, bool runExecutables = true,
+                       const QString &suggestedFileName = QString(), const QByteArray &asn = QByteArray());
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Open the given URL.
      *
@@ -318,10 +400,23 @@ public:
      * @param asn Application startup notification id, if any (otherwise "").
      * @return @c true on success, @c false on error
      * @since 5.31
+     * @deprecated since 5.71, use KIO::OpenUrlJob:
+     * @code
+     *   KIO::OpenUrlJob *job = new KIO::OpenUrlJob(url, mimetype);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setRunExecutables(runExecutables);
+     *   job->setDeleteTemporaryFile(...); // depending on the old RunFlags
+     *   job->setSuggestedFileName(suggestedFileName);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::OpenUrlJob, see API docs for a code sample")
     static bool runUrl(const QUrl &url, const QString &mimetype, QWidget *window, RunFlags flags,
                        const QString &suggestedFileName = QString(), const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Run the given shell command and notifies KDE of the starting
      * of the application. If the program to be called doesn't exist,
@@ -337,9 +432,19 @@ public:
      * a command like "kwrite file.txt" finds file.txt from the right place
      *
      * @return @c true on success, @c false on error
+     * @deprecated since 5.71, use KIO::CommandLauncherJob
+     * @code
+     *   KIO::CommandLauncherJob *job = new KIO::CommandLauncherJob(cmd);
+     *   job->setUiDelegate(new KDialogJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setWorkingDirectory(workingDirectory);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::CommandLauncherJob, see API docs for a code sample")
     static bool runCommand(const QString &cmd, QWidget *window, const QString &workingDirectory = QString());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Same as the other runCommand(), but it also takes the name of the
      * binary, to display an error message in case it couldn't find it.
@@ -351,50 +456,91 @@ public:
      * @param window The top-level widget of the app that invoked this object.
      * @param asn Application startup notification id, if any (otherwise "").
      * @return @c true on success, @c false on error
+     * @deprecated since 5.71, use KIO::CommandLauncherJob
+     * @code
+     *   KIO::CommandLauncherJob *job = new KIO::CommandLauncherJob(cmd);
+     *   job->setUiDelegate(new KDialogJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setWorkingDirectory(workingDirectory);
+     *   job->setExecutable(execName);
+     *   job->setIcon(icon);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::CommandLauncherJob, see API docs for a code sample")
     static bool runCommand(const QString &cmd, const QString &execName,
                            const QString &icon, QWidget *window, const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Overload that also takes a working directory, so that a command like
      * "kwrite file.txt" finds file.txt from the right place.
      * @param workingDirectory the working directory for the started process. The default
      *                         (if passing an empty string) is the user's document path.
      * @since 4.4
+     * @deprecated since 5.71, use KIO::CommandLauncherJob instead
+     * @code
+     *   KIO::CommandLauncherJob *job = new KIO::CommandLauncherJob(cmd);
+     *   job->setUiDelegate(new KDialogJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   job->setWorkingDirectory(workingDirectory);
+     *   job->setExecutable(execName);
+     *   job->setIcon(icon);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::CommandLauncherJob, see API docs for a code sample")
     static bool runCommand(const QString &cmd, const QString &execName,
                            const QString &icon, QWidget *window,
                            const QByteArray &asn, const QString &workingDirectory);
-    // TODO KDE5: merge the above with 5-args runCommand, using QString()
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 71)
     /**
      * Display the Open-With dialog for those URLs, and run the chosen application.
-     * @param lst the list of applications to run
+     * @param lst the list of URLs to open
      * @param window The top-level widget of the app that invoked this object.
      * @param tempFiles if true and lst are local files, they will be deleted
      *        when the application exits.
      * @param suggestedFileName see setSuggestedFileName
      * @param asn Application startup notification id, if any (otherwise "").
      * @return false if the dialog was canceled
+     * @deprecated since 5.71, use KIO::ApplicationLauncherJob with no service argument (or a null service)
+     * @code
+     *   KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob();
+     *   job->setUrls(urls);
+     *   job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, window));
+     *   if (tempFiles) {
+     *       job->setRunFlags(KIO::ApplicationLauncherJob::DeleteTemporaryFiles);
+     *   }
+     *   job->setSuggestedFileName(suggestedFileName);
+     *   job->setStartupId(asn);
+     *   job->start();
+     * @endcode
      */
+    KIOWIDGETS_DEPRECATED_VERSION(5, 71, "Use KIO::OpenUrlJob, see API docs for a code sample")
     static bool displayOpenWithDialog(const QList<QUrl> &lst, QWidget *window,
                                       bool tempFiles = false, const QString &suggestedFileName = QString(),
                                       const QByteArray &asn = QByteArray());
+#endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 0)
     /**
      * Quotes a string for the shell.
      * An empty string will @em not be quoted.
      *
-     * @deprecated Use KShell::quoteArg() instead. @em Note that this function
+     * @param str the string to quote. The quoted string will be written here
+     *
+     * @deprecated Since 4.0, use KShell::quoteArg() instead. @em Note that this function
      *  behaves differently for empty arguments and returns the result
      *  differently.
-     *
-     * @param str the string to quote. The quoted string will be written here
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    static KIOWIDGETS_DEPRECATED void shellQuote(QString &str);
+    KIOWIDGETS_DEPRECATED_VERSION(4, 0, "Use KShell::quoteArg(...)")
+    static void shellQuote(QString &str);
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Processes a Exec= line as found in .desktop files.
      * @param _service the service to extract information from.
@@ -406,12 +552,13 @@ public:
      * @return a list of arguments suitable for QProcess.
      * @deprecated since 5.0, use KIO::DesktopExecParser
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    static KIOWIDGETS_DEPRECATED QStringList processDesktopExec(const KService &_service, const QList<QUrl> &_urls,
+    KIOWIDGETS_DEPRECATED_VERSION(5, 0, "Use KIO::DesktopExecParser")
+    static QStringList processDesktopExec(const KService &_service, const QList<QUrl> &_urls,
             bool tempFiles = false,
             const QString &suggestedFileName = QString());
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * Given a full command line (e.g. the Exec= line from a .desktop file),
      * extract the name of the binary being run.
@@ -421,15 +568,15 @@ public:
      * @deprecated since 5.0, use KIO::DesktopExecParser::executableName if removePath was true,
      * or KIO::DesktopExecParser::executablePath if removePath was false.
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
+    KIOWIDGETS_DEPRECATED_VERSION(5, 0, "See API docs")
     static QString binaryName(const QString &execLine, bool removePath);
 #endif
 
     /**
-     * Returns whether @p serviceType refers to an executable program instead
+     * Returns whether @p mimeType refers to an executable program instead
      * of a data file.
      */
-    static bool isExecutable(const QString &serviceType);
+    static bool isExecutable(const QString &mimeType);
 
     /**
      * Returns whether the @p url of @p mimetype is executable.
@@ -473,7 +620,7 @@ protected Q_SLOTS:
      * This slot is called whenever the internal timer fired,
      * in order to move on to the next step.
      */
-    void slotTimeout(); // KDE5: rename to slotNextStep() or something like that
+    void slotTimeout();
 
     /**
      * This slot is called when the scan job is finished.
@@ -586,37 +733,41 @@ protected:
      */
     KIO::Job *job();
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 4)
     /**
      * Returns the timer object.
-     * @deprecated setFinished(true) now takes care of the timer().start(0),
+     * @deprecated Since 4.4. setFinished(true) now takes care of the timer().start(0),
      * so this can be removed.
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED QTimer &timer();
+    KIOWIDGETS_DEPRECATED_VERSION(4, 4, "See API docs")
+    QTimer &timer();
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 1)
     /**
      * Indicate that the next action is to scan the file.
-     * @deprecated not useful in public API
+     * @deprecated Since 4.1. Not useful in public API
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED void setDoScanFile(bool scanFile);
+    KIOWIDGETS_DEPRECATED_VERSION(4, 1, "Do not use")
+    void setDoScanFile(bool scanFile);
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 1)
     /**
      * Returns whether the file shall be scanned.
-     * @deprecated not useful in public API
+     * @deprecated Since 4.1. Not useful in public API
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED bool doScanFile() const;
+    KIOWIDGETS_DEPRECATED_VERSION(4, 1, "Do not use")
+    bool doScanFile() const;
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 1)
     /**
      * Sets whether it is a directory.
-     * @deprecated typo in the name, and not useful as a public method
+     * @deprecated Since 4.1. Typo in the name, and not useful as a public method
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED void setIsDirecory(bool isDirectory);
+    KIOWIDGETS_DEPRECATED_VERSION(4, 1, "Do not use")
+    void setIsDirecory(bool isDirectory);
 #endif
 
     /**
@@ -624,18 +775,20 @@ protected:
      */
     bool isDirectory() const;
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 1)
     /**
-     * @deprecated not useful in public API
+     * @deprecated Since 4.1. Not useful in public API
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED void setInitializeNextAction(bool initialize);
+    KIOWIDGETS_DEPRECATED_VERSION(4, 1, "Do not use")
+    void setInitializeNextAction(bool initialize);
 #endif
 
+#if KIOWIDGETS_ENABLE_DEPRECATED_SINCE(4, 1)
     /**
-     * @deprecated not useful in public API
+     * @deprecated Since 4.1. Not useful in public API
      */
-#ifndef KIOWIDGETS_NO_DEPRECATED
-    KIOWIDGETS_DEPRECATED bool initializeNextAction() const;
+    KIOWIDGETS_DEPRECATED_VERSION(4, 1, "Do not use")
+    bool initializeNextAction() const;
 #endif
 
     /**
@@ -644,8 +797,9 @@ protected:
     bool isLocalFile() const;
 
 private:
-    class KRunPrivate;
+    friend class KRunPrivate;
     KRunPrivate *const d;
 };
+#endif
 
 #endif
