@@ -1,17 +1,14 @@
 /*
- This file is or will be part of KDE desktop environment
+    This file is or will be part of KDE desktop environment
+    SPDX-FileCopyrightText: 1999 Matt Koss <koss@miesto.sk>
 
- Copyright 1999 Matt Koss <koss@miesto.sk>
-
- It is licensed under GPL version 2.
-
- If it is part of KDE libraries than this file is licensed under
- LGPL version 2.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "kioslavetest.h"
 
 #include <QApplication>
+#include <QLocale>
 #include <QLayout>
 #include <QMessageBox>
 #include <QDir>
@@ -23,14 +20,14 @@
 
 #include <qplatformdefs.h>
 
-#include <kjobuidelegate.h>
+#include <KJobUiDelegate>
 #include <kio/job.h>
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
 #include <kprotocolinfo.h>
 #include <QTimer>
-#include <qcommandlineparser.h>
-#include <qcommandlineoption.h>
+#include <QCommandLineParser>
+#include <QCommandLineOption>
 
 using namespace KIO;
 
@@ -249,7 +246,7 @@ void KioslaveTest::startJob()
         break;
 
     case Stat:
-        myJob = KIO::stat(src, KIO::StatJob::SourceSide, 2);
+        myJob = KIO::statDetails(src, KIO::StatJob::SourceSide);
         break;
 
     case Get:
@@ -388,15 +385,15 @@ void KioslaveTest::printUDSEntry(const KIO::UDSEntry &entry)
             break;
         case KIO::UDSEntry::UDS_CREATION_TIME:
             timestamp = QDateTime::fromSecsSinceEpoch(entry.numberValue(*it));
-            qDebug() << "CreationTime: " << timestamp.toString(Qt::SystemLocaleShortDate);
+            qDebug() << "CreationTime: " << QLocale().toString(timestamp, QLocale::ShortFormat);
             break;
         case KIO::UDSEntry::UDS_MODIFICATION_TIME:
             timestamp = QDateTime::fromSecsSinceEpoch(entry.numberValue(*it));
-            qDebug() << "ModificationTime: " << timestamp.toString(Qt::SystemLocaleShortDate);
+            qDebug() << "ModificationTime: " << QLocale().toString(timestamp, QLocale::ShortFormat);
             break;
         case KIO::UDSEntry::UDS_ACCESS_TIME:
             timestamp = QDateTime::fromSecsSinceEpoch(entry.numberValue(*it));
-            qDebug() << "AccessTime: " << timestamp.toString(Qt::SystemLocaleShortDate);
+            qDebug() << "AccessTime: " << QLocale().toString(timestamp, QLocale::ShortFormat);
             break;
         }
     }
